@@ -12,11 +12,15 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
  * License for more details.
  *
- * RCSID $Id: ttoaddr.c,v 1.11 2004-04-09 18:00:37 mcr Exp $
+ * RCSID $Id: ttoaddr.c,v 1.13 2005/08/05 17:36:24 mcr Exp $
  */
 #include "internal.h"
 #include "openswan.h"
 
+#if defined(__CYGWIN32__)
+#define gethostbyname2(X,Y) gethostbyname(X)
+#endif
+
 /*
  * Legal ASCII characters in a domain name.  Underscore technically is not,
  * but is a common misunderstanding.  Non-ASCII characters are simply
@@ -38,23 +42,14 @@ static err_t getpiece(const char **, const char *, unsigned *);
  - ttoaddr - convert text name or dotted-decimal address to binary address
  */
 err_t				/* NULL for success, else string literal */
-ttoaddr(src, srclen, af, dst)
-const char *src;
-size_t srclen;			/* 0 means "apply strlen" */
-int af;				/* address family */
-ip_address *dst;
+ttoaddr_base(const char *src,
+	     size_t srclen,		/* 0 means "apply strlen" */
+	     int    af,			/* address family */
+	     int   *allnumericfailed,
+	     ip_address *dst)
 {
 	err_t oops;
 #	define	HEXLEN	10	/* strlen("0x11223344") */
-	int nultermd;
-
-	if (srclen == 0) {
-		srclen = strlen(src);
-		if (srclen == 0)
-			return "empty string";
-		nultermd = 1;
-	} else
-		nultermd = 0;	/* at least, not *known* to be terminated */
 
 	switch (af) {
 	case AF_INET:
@@ -92,7 +87,8 @@ ip_address *dst;
 			return oops;		/* probably meant as d-d */
 	}
 
-	return tryname(src, srclen, nultermd, af, dst);
+	*allnumericfailed=1;
+	return "not numeric";
 }
 
 /*
@@ -182,8 +178,11 @@ ip_address *dst;
 	}
 
 	h = gethostbyname2(cp, af);
+#if !defined(__CYGWIN32__)
+	/* like, windows even has an /etc/networks? */
 	if (h == NULL && af == AF_INET)
 		ne = getnetbyname(cp);
+#endif
 	if (p != namebuf)
 		FREE(p);
 	if (h == NULL && ne == NULL)
@@ -348,6 +347,7 @@ ip_address *dst;
 	}
 
 	gapat = -1;
+	piece=0;
 	for (i = 0; i < NPIECES && src < stop; i++) {
 		oops = getpiece(&src, stop, &piece);
 		if (oops != NULL && *oops == ':') {	/* empty field */
@@ -424,3 +424,172 @@ unsigned *retp;			/* return-value pointer */
 	*retp = ret;
 	return NULL;
 }
+
+err_t				/* NULL for success, else string literal */
+ttoaddr(const char *src,
+	size_t srclen,	/* 0 means "apply strlen" */
+	int af,		/* address family */
+	ip_address *dst)
+{
+	int nultermd;
+	int numfailed=0;
+	err_t err;
+	if (srclen == 0) {
+		srclen = strlen(src);
+		if (srclen == 0)
+			return "empty string";
+		nultermd = 1;
+	} else
+		nultermd = 0;	/* at least, not *known* to be terminated */
+
+	err = ttoaddr_base(src, srclen, af, &numfailed, dst);
+
+	if(err && numfailed) {
+		err = NULL;
+		if(af == 0) {
+			err = tryname(src, srclen, nultermd, AF_INET6, dst);
+			if(err) {
+				af=AF_INET;
+			}
+		}
+		if(err) {
+			err = tryname(src, srclen, nultermd, af, dst);
+		}
+		return err;
+	}
+	
+	return err;
+}
+
+err_t				/* NULL for success, else string literal */
+ttoaddr_num(const char *src,
+	    size_t srclen,	/* 0 means "apply strlen" */
+	    int af,		/* address family */
+	    ip_address *dst)
+{
+	int numfailed=0;
+
+	if (srclen == 0) {
+		srclen = strlen(src);
+		if (srclen == 0)
+			return "empty string";
+	} 
+
+	err_t err = ttoaddr_base(src, srclen, af, &numfailed, dst);
+	return err;
+}
+
+#ifdef TTOADDR_MAIN
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+
+void regress(void);
+
+int
+main(int argc, char *argv[])
+{
+	if (argc < 2) {
+		fprintf(stderr, "Usage: %s {addr|net/mask|begin...end|-r}\n",
+								argv[0]);
+		exit(2);
+	}
+
+	if (strcmp(argv[1], "-r") == 0) {
+		regress();
+		fprintf(stderr, "regress() returned?!?\n");
+		exit(1);
+	}
+	exit(0);
+}
+
+struct rtab {
+	char *input;
+	char  numonly;
+        char  format;
+	char  expectfailure;
+	char *output;			/* NULL means error expected */
+} rtab[] = {
+	{"1.2.3.0",	0,	0,   0,       "1.2.3.0"},
+	{"1:2::3:4",    0,      0,   0,       "1:2::3:4"},
+	{"1:2::3:4",    0,      'Q', 0,       "1:2:0:0:0:0:3:4"},
+	{"1:2:0:0:3:4:0:0", 0,    0, 0,       "1:2::3:4:0:0"},
+	{"www.openswan.org", 0,   0, 0,       "193.110.157.129"},
+	{"www.openswan.org", 1,   0, 'F',     "1.2.3.4"},
+	{"1.2.3.4",         0,   'r',0,       "4.3.2.1.IN-ADDR.ARPA."},
+ 	/*                                   0 1 2 3 4 5 6 7 8 9 a b c d e f 0 1 2 3 4 5 6 7 8 9 a b c d e f */
+	{"1:2::3:4",        0,   'r',0,     "4.0.0.0.3.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.2.0.0.0.1.0.0.0.IP6.ARPA."},
+	{NULL,		    0,  0, 0, NULL}
+};
+
+void
+regress()
+{
+	struct rtab *r;
+	int status = 0;
+	ip_address a;
+	char in[100];
+	char buf[100];
+	const char *oops;
+	size_t n;
+	int count=0;
+
+	for (r = rtab; r->input != NULL; r++) {
+		++count;
+		memset(&a, 0, sizeof(a));
+		strcpy(in, r->input);
+
+		if(r->numonly) {
+			/* convert it *to* internal format (no DNS) */
+			oops = ttoaddr_num(in, strlen(in), 0, &a);
+		} else {
+			/* convert it *to* internal format */
+			oops = ttoaddr(in, strlen(in), 0, &a);
+		}
+
+		if(r->expectfailure && oops==NULL) {
+			printf("%u: '%s' expected failure, but it succeeded\n",
+			       count, r->input);
+			status++;
+			continue;
+		}
+			
+		if(oops) {
+			if(r->expectfailure) continue;
+			printf("%u: '%s' failed to parse: %s\n",
+			       count, r->input, oops);
+			status++;;
+			continue;
+		}
+
+		/* now convert it back */
+
+		n = addrtot(&a, r->format, buf, sizeof(buf));
+
+		if (n == 0 && r->output == NULL)
+			{}		/* okay, error expected */
+		
+		else if (n == 0) {
+			printf("`%s' atoasr failed\n", r->input);
+			status++;
+			
+		} else if (r->output == NULL) {
+			printf("`%s' atoasr succeeded unexpectedly '%c'\n",
+							r->input, r->format);
+			status++;
+		} else {
+		  if (strcasecmp(r->output, buf) != 0) {
+		    printf("`%s' '%u' gave `%s', expected `%s'\n",
+			   r->input, r->format, buf, r->output);
+		    status++;
+		  }
+		}
+	}
+	exit(status);
+}
+
+#endif /* ADDRTOT_MAIN */
+
