@@ -84,11 +84,12 @@ static const char *const timer_event_name[] = {
 	"EVENT_DPD_TIMEOUT",
 	"EVENT_LOG_DAILY",
 	"EVENT_CRYPTO_FAILED",
-	"EVENT_PENDING_PHASE2"
+	"EVENT_PENDING_PHASE2",
+	"EVENT_v2_RETRANSMIT"
     };
 
 enum_names timer_event_names =
-    { EVENT_NULL, EVENT_PENDING_PHASE2, timer_event_name, NULL };
+    { EVENT_NULL, EVENT_v2_RETRANSMIT, timer_event_name, NULL };
 
 /* State of exchanges */
 static const char *const state_name[] = {
