@@ -141,6 +141,7 @@ static const struct state_v2_microcode state_microcode_table[] = {
       .flags = SMF2_INITIATOR|SMF2_STATENEEDED|SMF2_REPLY,
       .processor  = ikev2parent_inR1outI2,
       .recv_type  = ISAKMP_v2_SA_INIT,
+      .timeout_event = EVENT_SA_REPLACE,
     },
 
     { .state      = STATE_PARENT_I2,
@@ -162,6 +163,7 @@ static const struct state_v2_microcode state_microcode_table[] = {
       .flags = SMF2_RESPONDER|SMF2_STATENEEDED|SMF2_REPLY,
       .processor  = ikev2parent_inI2outR2,
       .recv_type  = ISAKMP_v2_AUTH,
+      .timeout_event = EVENT_SA_REPLACE,
     },
 
     /* last entry */
@@ -638,11 +640,6 @@ static void success_v2_state_transition(struct msg_digest **mdp)
     st->st_state = svm->next_state;
     w = RC_NEW_STATE + st->st_state;    
 
-    /* Delete previous retransmission event.
-     * New event will be scheduled below.
-     */
-    delete_event(st);
-
     ikev2_update_counters(md);
 
 
@@ -657,7 +654,7 @@ static void success_v2_state_transition(struct msg_digest **mdp)
 	sadetails[0]='\0';
 
 	/* document IPsec SA details for admin's pleasure */
-	if(IS_CHILD_SA_ESTABLISHED(st->st_state))
+	if(IS_CHILD_SA_ESTABLISHED(st))
 	{
 	    char usubl[128], usubh[128];
 	    char tsubl[128], tsubh[128];
@@ -675,7 +672,7 @@ static void success_v2_state_transition(struct msg_digest **mdp)
 	    fmt_isakmp_sa_established(st, sadetails,sizeof(sadetails));
 	}
 	
-	if (IS_CHILD_SA_ESTABLISHED(st->st_state))
+	if (IS_CHILD_SA_ESTABLISHED(st))
 	{
 	    /* log our success */
 	    w = RC_SUCCESS;
@@ -689,14 +686,14 @@ static void success_v2_state_transition(struct msg_digest **mdp)
 	       , sadetails);
     }
 
-    /* free previous transmit packet */
-    freeanychunk(st->st_tpacket);
-
     /* if requested, send the new reply packet */
     if (svm->flags & SMF2_REPLY)
     {
 	char buf[ADDRTOT_BUF];
 
+	/* free previously transmitted packet */
+	freeanychunk(st->st_tpacket);
+
 	if(nat_traversal_enabled) {
 	    /* adjust our destination port if necessary */
 	    nat_traversal_change_port_lookup(md, st);
@@ -741,6 +738,98 @@ static void success_v2_state_transition(struct msg_digest **mdp)
 	    release_whack(pst);
 	}
     }
+
+    /* Schedule for whatever timeout is specified */
+    {
+	time_t delay;
+	enum event_type kind = svm->timeout_event;
+	bool agreed_time = FALSE;
+	struct connection *c = st->st_connection;
+
+	switch (kind)
+	{
+	case EVENT_SA_REPLACE:	/* SA replacement event */
+	    if (IS_PARENT_SA(st))
+	    {
+		/* Note: we will defer to the "negotiated" (dictated)
+		 * lifetime if we are POLICY_DONT_REKEY.
+		 * This allows the other side to dictate
+		 * a time we would not otherwise accept
+		 * but it prevents us from having to initiate
+		 * rekeying.  The negative consequences seem
+		 * minor.
+		 */
+		delay = c->sa_ike_life_seconds;
+	    }
+	    else
+	    {
+		/* Delay is what the user said, no negotiation.
+		 */
+		delay = c->sa_ipsec_life_seconds;
+	    }
+	    
+	    /* By default, we plan to rekey.
+	     *
+	     * If there isn't enough time to rekey, plan to
+	     * expire.
+	     *
+	     * If we are --dontrekey, a lot more rules apply.
+	     * If we are the Initiator, use REPLACE_IF_USED.
+	     * If we are the Responder, and the dictated time
+	     * was unacceptable (too large), plan to REPLACE
+	     * (the only way to ratchet down the time).
+	     * If we are the Responder, and the dictated time
+	     * is acceptable, plan to EXPIRE.
+	     *
+	     * Important policy lies buried here.
+	     * For example, we favour the initiator over the
+	     * responder by making the initiator start rekeying
+	     * sooner.  Also, fuzz is only added to the
+	     * initiator's margin.
+	     *
+	     * Note: for ISAKMP SA, we let the negotiated
+	     * time stand (implemented by earlier logic).
+	     */
+	    if (agreed_time
+		&& (c->policy & POLICY_DONT_REKEY))
+	    {
+		kind = (svm->flags & SMF2_INITIATOR)
+		    ? EVENT_SA_REPLACE_IF_USED
+		    : EVENT_SA_EXPIRE;
+	    }
+	    if (kind != EVENT_SA_EXPIRE)
+	    {
+		unsigned long marg = c->sa_rekey_margin;
+		
+		if (svm->flags & SMF2_INITIATOR)
+		    marg += marg
+			* c->sa_rekey_fuzz / 100.E0
+			* (rand() / (RAND_MAX + 1.E0));
+		else
+		    marg /= 2;
+		
+		if ((unsigned long)delay > marg)
+		{
+			    delay -= marg;
+			    st->st_margin = marg;
+		}
+		else
+		{
+		    kind = EVENT_SA_EXPIRE;
+		}
+	    }
+	    delete_event(st);
+	    event_schedule(kind, delay, st);
+	    break;
+	    
+	case EVENT_NULL:
+	    break;
+
+	case EVENT_REINIT_SECRET:	/* Refresh cookie secret */
+	default:
+	    bad_case(kind);
+	}
+    }
 }
 
 void complete_v2_state_transition(struct msg_digest **mdp
@@ -750,6 +839,7 @@ void complete_v2_state_transition(struct msg_digest **mdp
     //const struct state_v2_microcode *svm=md->svm;
     struct state *st;
     enum state_kind from_state;
+    const char *from_state_name;
 
     cur_state = st = md->st;	/* might have changed */
 
@@ -828,8 +918,15 @@ void complete_v2_state_transition(struct msg_digest **mdp
 	/* FALL THROUGH ... */
 
     case STF_FAIL:
+	if(st) {
+	    from_state_name = enum_name(&state_names, st->st_state);
+	} else {
+	    from_state_name = "no-state";
+	}
+	    
 	whack_log(RC_NOTIFICATION + md->note
-		  , "%s: %s", enum_name(&state_names, st->st_state)
+		  , "%s: %s"
+		  , from_state_name
 		  , enum_name(&ipsec_notification_names, md->note));
 
 #if 0
@@ -838,10 +935,9 @@ void complete_v2_state_transition(struct msg_digest **mdp
 	}
 #endif
 	
-    	from_state   = st->st_state;
 	DBG(DBG_CONTROL,
 	    DBG_log("state transition function for %s failed: %s"
-		    , enum_name(&state_names, from_state)
+		    , from_state_name
 		    , enum_name(&ipsec_notification_names, md->note)));
     }
 }
