@@ -114,6 +114,7 @@ ikev2parent_outI1(int whack_sock
     st->st_state = STATE_PARENT_I1;
     st->st_msgid_lastack = INVALID_MSGID;
     st->st_msgid_nextuse = 0;
+    st->st_try   = try;
 
     if (HAS_IPSEC_POLICY(policy))
 	add_pending(dup_any(whack_sock), st, c, policy, 1
@@ -426,10 +427,13 @@ ikev2_parent_outI1_common(struct msg_digest *md
 
     /* let TCL hack it before we mark the length and copy it */
     TCLCALLOUT("v2_avoidEmitting", st, st->st_connection, md);
+
+    freeanychunk(st->st_tpacket);
     clonetochunk(st->st_tpacket, md->reply.start, pbs_offset(&md->reply)
-       , "reply packet for ikev2_parent_outI1");
+		 , "reply packet for ikev2_parent_outI1_tail");
 
     /* save packet for later signing */
+    freeanychunk(st->st_firstpacket_me);
     clonetochunk(st->st_firstpacket_me, md->reply.start
 		 , pbs_offset(&md->reply), "saved first packet");
 
@@ -444,7 +448,7 @@ ikev2_parent_outI1_common(struct msg_digest *md
  tpm_ignore:
 #endif
     delete_event(st);
-    event_schedule(EVENT_RETRANSMIT, EVENT_RETRANSMIT_DELAY_0, st);
+    event_schedule(EVENT_v2_RETRANSMIT, EVENT_RETRANSMIT_DELAY_0, st);
 
     reset_cur_state();
     return STF_OK;
@@ -754,10 +758,12 @@ ikev2_parent_inI1outR1_tail(struct pluto_crypto_req_cont *pcrc
     TCLCALLOUT("v2_avoidEmitting", st, st->st_connection, md);
 
     /* keep it for a retransmit if necessary */
+    freeanychunk(st->st_tpacket);
     clonetochunk(st->st_tpacket, md->reply.start, pbs_offset(&md->reply)
-		 , "reply packet for ikev2_parent_outI1");
+		 , "reply packet for ikev2_parent_inI1outR1_tail")
 
     /* save packet for later signing */
+    freeanychunk(st->st_firstpacket_me);
     clonetochunk(st->st_firstpacket_me, md->reply.start
 		 , pbs_offset(&md->reply), "saved first packet");
 
@@ -824,6 +830,12 @@ stf_status ikev2parent_inR1outI2(struct msg_digest *md)
 
     DBG(DBG_CONTROLMORE
 	, DBG_log("ikev2 parent inR1: calculating g^{xy} in order to send I2"));
+
+    if(st->st_gr.len == 0) {
+	/* Remote end did not send us Gr in R1 - likely a NOTIFY message */
+	openswan_log("No responder Gr found in R1 packet");
+	return PAYLOAD_MALFORMED;
+    }
  
     /* KE in */
     keyex_pbs = &md->chain[ISAKMP_NEXT_v2KE]->pbs;
@@ -1100,8 +1112,14 @@ stf_status ikev2_decrypt_msg(struct msg_digest *md
 	
 	init_pbs(&md->clr_pbs, encstart, enclen - (padlen+1), "cleartext");
     }
-    
-    ikev2_process_payloads(md, &md->clr_pbs, st->st_state, np);
+
+    { stf_status ret;
+	ret = ikev2_process_payloads(md, &md->clr_pbs, st->st_state, np);
+	if(ret != STF_OK) {
+	    return ret;
+	}
+    }
+	    
     return STF_OK;
 }
 
@@ -1184,6 +1202,10 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
     md->st = st;
     md->pst= pst;
 
+    /* parent had crypto failed, replace it with rekey! */
+    delete_event(pst);
+    event_schedule(EVENT_SA_REPLACE, c->sa_ike_life_seconds, pst);
+
     /* need to force parent state to I2 */
     pst->st_state = STATE_PARENT_I2;
 
@@ -1281,7 +1303,7 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 	if(send_cert) {
 	    stf_status certstat = ikev2_send_cert( st, md
 	    					   , INITIATOR
-						   ,ISAKMP_NEXT_v2AUTH
+						   , ISAKMP_NEXT_v2AUTH
 						   , &e_pbs_cipher);
 	    if(certstat != STF_OK) return certstat;
 	}
@@ -1289,20 +1311,19 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 
     /* send out the AUTH payload */
     {
+	lset_t policy;
+	struct connection *c0= first_pending(pst, &policy,&st->st_whack_sock);
+	unsigned int np = (c0 ? ISAKMP_NEXT_v2SA : ISAKMP_NEXT_NONE);
 	stf_status authstat = ikev2_send_auth(c, st
-					      , INITIATOR, ISAKMP_NEXT_v2SA
+					      , INITIATOR
+					      , np
 					      , idhash, &e_pbs_cipher);
 	if(authstat != STF_OK) return authstat;
-    }
-
-    /*
-     * now, find an eligible child SA from the pending list, and emit
-     * SA2i, TSi and TSr for it.
-     */
-    {
-	lset_t policy;
-	struct connection *c0 = first_pending(pst, &policy,&st->st_whack_sock);
 
+	/*
+	 * now, find an eligible child SA from the pending list, and emit
+	 * SA2i, TSi and TSr for it.
+	 */
 	if(c0) {
 	    st->st_connection = c0;
 	    ikev2_emit_ipsec_sa(md,&e_pbs_cipher,ISAKMP_NEXT_v2TSi,c0, policy);
@@ -1311,6 +1332,8 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 	    st->st_ts_that = ikev2_subnettots(&c0->spd.that);
 	    
 	    ikev2_calc_emit_ts(md, &e_pbs_cipher, INITIATOR, c0, policy);
+	} else {
+	    openswan_log("no pending SAs found, PARENT SA keyed only");
 	}
     }
 
@@ -1344,10 +1367,15 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
     /* keep it for a retransmit if necessary, but on initiator
      * we never do that, but send_packet() uses it.
      */
+    freeanychunk(pst->st_tpacket);
     clonetochunk(pst->st_tpacket, md->reply.start, pbs_offset(&md->reply)
 		 , "reply packet for ikev2_parent_outI1");
 
-    /* note: retransimission is driven by initiator */
+    /*
+     * Delete previous retransmission event.
+     */
+    delete_event(st);
+    event_schedule(EVENT_v2_RETRANSMIT, EVENT_RETRANSMIT_DELAY_0, st);
 
     return STF_OK;
     
@@ -1610,7 +1638,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	get_rnd_bytes(iv, ivsize);
 	
 	/* note where cleartext starts */
-	init_pbs(&e_pbs_cipher, e_pbs.cur, e_pbs.roof - e_pbs.cur, "cleartext");
+	init_pbs(&e_pbs_cipher, e_pbs.cur, e_pbs.roof-e_pbs.cur, "cleartext");
 	e_pbs_cipher.container = &e_pbs;
 	e_pbs_cipher.desc = NULL;
 	e_pbs_cipher.cur = e_pbs.cur;
@@ -1672,13 +1700,14 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	    if(certstat != STF_OK) return certstat;
     	} 
 
-	/* authentication good, see if there is a child SA available */
+	/* authentication good, see if there is a child SA being proposed */
 	if(md->chain[ISAKMP_NEXT_v2SA] == NULL
 	   || md->chain[ISAKMP_NEXT_v2TSi] == NULL
 	   || md->chain[ISAKMP_NEXT_v2TSr] == NULL) {
 	    
 	    /* initiator didn't propose anything. Weird. Try unpending out end. */
 	    /* UNPEND XXX */
+	    openswan_log("No CHILD SA proposals received.");
 	    np = ISAKMP_NEXT_NONE;
 	} else {
 	    np = ISAKMP_NEXT_v2SA;
@@ -1727,8 +1756,9 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
     TCLCALLOUT("v2_avoidEmitting", st, st->st_connection, md);
 
     /* keep it for a retransmit if necessary */
+    freeanychunk(st->st_tpacket);
     clonetochunk(st->st_tpacket, md->reply.start, pbs_offset(&md->reply)
-		 , "reply packet for ikev2_parent_outI1");
+		 , "reply packet for ikev2_parent_inI2outR2_tail");
 
     /* note: retransimission is driven by initiator */
 
@@ -1865,6 +1895,11 @@ stf_status ikev2parent_inR2(struct msg_digest *md)
 	|| md->chain[ISAKMP_NEXT_v2TSi] == NULL
 	|| md->chain[ISAKMP_NEXT_v2TSr] == NULL) {
 	/* not really anything to here... but it would be worth unpending again */
+
+	/*
+	 * Delete previous retransmission event.
+	 */
+	delete_event(st);
 	return STF_OK;
     }
 
@@ -1887,6 +1922,11 @@ stf_status ikev2parent_inR2(struct msg_digest *md)
     if(!install_ipsec_sa(st, TRUE))
 	return STF_FATAL;
 
+    /*
+     * Delete previous retransmission event.
+     */
+    delete_event(st);
+
     return STF_OK;
     
 }
