@@ -277,7 +277,18 @@ prerunsetup() {
 	REPORT_NAME=${TESTNAME}${KLIPS_MODULE}
     else
 	HOST_START=${HOST_START-$POOLSPACE/$TESTHOST/start.sh}
-	EAST_START=${EAST_START-$POOLSPACE/$EASTHOST/start.sh}
+	if [ -n "$EAST_NETKEY" ] 
+	 then
+	  EAST_START=${EAST_START-$POOLSPACE/$EASTHOST/start-netkey.sh}
+	else 
+	  EAST_START=${EAST_START-$POOLSPACE/$EASTHOST/start.sh}
+	fi
+	if [ -n "$WEST_NETKEY" ] 
+	 then
+	  WEST_START=${WEST_START-$POOLSPACE/$WESTHOST/start-netkey.sh}
+	else 
+	  WEST_START=${WEST_START-$POOLSPACE/$WESTHOST/start.sh}
+	fi
 	WEST_START=${WEST_START-$POOLSPACE/$WESTHOST/start.sh}
 	REPORT_NAME=${TESTNAME}
     fi
